library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity lab3 is
    port (
	--
	-- Deklaracije vanjskih sucelja sklopa: smjer i tip signala
	--
	clk_25m: in std_logic;
	btn_left, btn_right: in std_logic;
	rs232_tx: out std_logic;
	led: out std_logic_vector(7 downto 0)
    );
end lab3;

architecture x of lab3 is
    --
    -- Deklaracije internih signala: samo tip signala, bez smjera
    --
    signal pulse_left, pulse_right: std_logic;
    signal q: std_logic_vector(3 downto 0);
    signal sum: std_logic_vector(3 downto 0);
    signal ascii: std_logic_vector(7 downto 0);

begin
    --
    -- Instanciranje paralelno-serijskog pretvornika (RS232)
    --
    I_ser: entity work.ser_tx
    port map (
	clk => clk_25m,
	write => pulse_right,
	d7 => ascii(7), d6 => ascii(6), d5 => ascii(5), d4 => ascii(4),
	d3 => ascii(3), d2 => ascii(2), d1 => ascii(1), d0 => ascii(0),
	ser_out => rs232_tx
    );

    --
    -- Instanciranje modula za generiranje impulsa
    --
    I_pulse1: entity work.pulse
    port map (
        clk => clk_25m,
        btn => btn_left,
        pulse => pulse_left
    );

    I_pulse2: entity work.pulse
    port map (
        clk => clk_25m,
        btn => btn_right,
        pulse => pulse_right
    );

    --
    -- Instanciranje D bistabila
    --
    I_dff0: entity work.dff
    port map (
        clk => clk_25m,
	rst => pulse_left,
        we => pulse_right,
        d => sum(0),
	q => q(0)
    );

    I_dff1: entity work.dff
    port map (
        clk => clk_25m,
	rst => pulse_left,
        we => pulse_right,
        d => sum(1),
	q => q(1)
    );

    I_dff2: entity work.dff
    port map (
        clk => clk_25m,
	rst => pulse_left,
        we => pulse_right,
        d => sum(2),
	q => q(2)
    );

    I_dff3: entity work.dff
    port map (
        clk => clk_25m,
	rst => pulse_left,
        we => pulse_right,
        d => sum(3),
	q => q(3)
    );

    --
    -- Instanciranje zbrajala
    --
    I_add: entity work.adder
    port map (
	a3 => '0', a2 => '0', a1 => '0', a0 => '1',
	b3 => q(3), b2 => q(2), b1 => q(1), b0 => q(0),
	z3 => sum(3), z2 => sum(2), z1 => sum(1), z0 => sum(0)
    );

    --
    -- Prospoj signala ascii na vanjske LED indikatore
    --
    led <= ascii;

    --
    -- Kombinacijska logika: formiranje ASCII kodnih rijeci
    --
    ascii(7) <= '0';
    ascii(6) <= '0';
    ascii(5) <= not q(3);
    ascii(4) <= not q(3);
    ascii(3) <= q(3) or (not q(2) and not q(1) and not q(0));
    ascii(2) <= (not q(2) and q(1)) or (not q(3) and not q(2) and q(0))
      or (q(2) and not q(1) and not q(0)) or (q(3) and not q(0));
    ascii(1) <= (not q(1) and q(0)) or (q(1) and not q(0));
    ascii(0) <= (q(3) and not q(0)) or (not q(3) and q(0));
end x;
