library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

entity ser_tx is
    generic (
	C_clk_freq_mhz: integer := 25;	-- clock frequency, in MHz
	C_baudrate: integer := 115200	-- in bits per second
    );
    port (
	clk: in std_logic;
	d0, d1, d2, d3, d4, d5, d6, d7: in std_logic;
	write: in std_logic;
	ser_out: out std_logic
    );
end ser_tx;

architecture x of ser_tx is
    constant C_baud_init: std_logic_vector(15 downto 0) :=
      std_logic_vector(to_unsigned(
	C_baudrate * 2**10 / 1000 * 2**10 / C_clk_freq_mhz / 1000, 16));

    -- baud * 16 impulse generator
    signal R_baudrate: std_logic_vector(15 downto 0) := C_baud_init;
    signal R_baudgen: std_logic_vector(16 downto 0);

    -- transmit logic
    signal R_tx_tickcnt: std_logic_vector(3 downto 0);
    signal R_tx_phase: std_logic_vector(3 downto 0);
    signal R_tx_ser: std_logic_vector(8 downto 0) := (others => '1');

    -- aux bits to byte
    signal byte: std_logic_vector(7 downto 0);

begin

    byte <= d7 & d6 & d5 & d4 & d3 & d2 & d1 & d0;

    --
    -- tx phases:
    --	"0000" idle
    --	"0001" start bit
    --	"0010".."1001" data bits
    --	"1010" stop bit
    --

    ser_out <= R_tx_ser(0);

    process(clk)
    begin
	if rising_edge(clk) then
	    -- initiate tx
	    if write = '1' then
		R_tx_phase <= x"1";
		R_tx_ser <= byte & '0';
	    end if;

	    -- baud generator
	    R_baudgen <= ('0' & R_baudgen(15 downto 0)) + ('0' & R_baudrate);

	    -- tx logic
	    if R_tx_phase /= x"0" and R_baudgen(16) = '1' then
		R_tx_tickcnt <= R_tx_tickcnt + 1;
		if R_tx_tickcnt = x"f" then
		    R_tx_ser <= '1' & R_tx_ser(8 downto 1);
		    R_tx_phase <= R_tx_phase + 1;
		    if R_tx_phase = x"a" then
			R_tx_phase <= x"0";
		    end if;
		end if;
	    end if;
	end if;
    end process;
end x;
